@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2000-2002,2005,2008,2010,2013 by Solar Designer.  See LICENSE.
+ * Copyright (c) 2000-2002,2005,2008,2010,2013,2016 by Solar Designer
+ * See LICENSE
  */
 
 #include <stdio.h>
@@ -204,8 +205,8 @@ char *passwdqc_random(const passwdqc_params_qc_t *params)
 	}
 
 out:
-	memset(bytes, 0, sizeof(bytes));
-	memset(output, 0, length);
+	_passwdqc_memzero(bytes, sizeof(bytes));
+	_passwdqc_memzero(output, length);
 
 	close(fd);
 
